#!/usr/bin/perl
#------------------------------------------------------------------------------
# /var/motion/motionmail.pl - create image montage and send it to e-mail address(es)
#
# Copyright (c) 2005-2010 Taco Scheltema, taco(at)scheltema(dot)org
# Copyright (c)      2011 The Eisfair Team, team(at)eisfair(dot)org
#
# Creation:     2005-09-01 Taco Scheltema
# Last Update:  $Id: motionmail.pl 29646 2012-01-24 19:50:20Z jed $
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version..
#------------------------------------------------------------------------------
# This script will create a montage from a collection of images and send it
# to a list of users defined in $start_path/motionmail.conf. The script is
# typically called from motion, the  motion detection software for linux:
# (http://www.lavrsen.dk/twiki/bin/view/Motion/WebHome)
#
# As a pre-requisite the following eisfair packages need to be installed:
#
# perl, perl_addons_01, imagemagick
#
# Usually the following entry need to be added to the motion package
# configuration:
#
# MOTION_DEF_ON_EVENTSTART='motionmail.pl -start -e %v -i %t -t %S -q'
# MOTION_DEF_ON_EVENTEND='motionmail.pl -end -e %v -i %t -q'
#------------------------------------------------------------------------------

use File::Basename;
use Getopt::Long;
use Pod::Usage;
use Sys::Hostname;
use POSIX qw/strftime/;

my $start_path   = dirname(__FILE__);
my $config_file  = "$start_path/motionmail.conf";
my $dir_path     = '';
my $help         = '';
my $instance     = '';
my $man          = '';
my $event        = '';
my $sub_dir      = '';
my $ver          = '';
my $quiet        = '';
my $dbg          = '';     # enable verbose output
$now             = time;
$version         = '0.13';
undef $err;

GetOptions ( 'help|h'    => \$help,
             'man'       => \$man,
             'e=s'       => \$event,
             'end'       => \$end,
             'i=i'       => \$instance,
             't=i'       => \$eseconds,
             'start'     => \$start,
             'version|V' => \$ver,
             'q'         => \$quiet,
             'debug'     => \$dbg   # option '-debug' will enable verbose
    );                              # output only used for development

if ((!$event)&&(!$man)&&(!$start)&&($ver)) { $help=1; }
if ($help) { pod2usage( -verbose =>  1) && exit; }
if ($man)  { pod2usage({-verbose =>  2, -output => \*STDOUT}); }
if ($ver)  { print "motionmail version $version\n"; exit; }

if (($start)&&($event)&&($instance)) {
    ### -start command ###
    $event_file  = "$start_path/events/event-$event\_camera-$instance";
    $config_file = "$start_path/motionmail-$instance.conf";

    if ($eseconds < 1325376001) {
        print "\n\tValue of parameter -t to low, should be greater than 1325376001!\n" if (($dbg)&&(!$quiet));
        print "\tsee 'motionmail.pl -man' for instructions.\n\n" if (!$quiet);
        exit;
    }

    if (!-e "$config_file") {
        print "\n\t$config_file not found!\n" if (($dbg)&&(!$quiet));
        print "\tsee 'motionmail.pl -man' for instructions.\n\n" if (!$quiet);
        exit;
    }

    if(!-e "$start_path/events") { mkdir "$start_path/events", 0755; }

    $excl = get_settings();

    if($excl) {
        # do not notify when the event was within an exclusion time block
        print "(".(get_date($now))[2].") Within exclusion time, not sending.\n" if (($dbg)&&(!$quiet));
        exit;
    } else {
        @time = localtime($eseconds);
        $sub_dir=strftime('%Y/%m/%d/%H/%M', @time);

        unlink("$event_file");
        open(FILE, '>>', $event_file) || die "Can't open event_file ($event_file) for writing!: $!\n"; 

        print FILE "dir_path=$setting{'file_dir'}/$sub_dir\n";
        print FILE "sub_dir=$sub_dir\n";
        print FILE "date=" . strftime('%d.%m.%Y', @time) . "\n";
        print FILE "time=" . strftime('%H:%M:%S', @time) . "\n";

        close(FILE);
        exit;
    }
} elsif (($end)&&($event)&&($instance)) {
    ### -end command ###
    $event_file  = "$start_path/events/event-$event\_camera-$instance";
    $config_file = "$start_path/motionmail-$instance.conf";

    if (!-e "$event_file") {
        print "\n\t$event_file not found!\n" if (($dbg)&&(!$quiet));
        print "\tsee 'motionmail.pl -man' for instructions.\n\n" if (!$quiet);
        exit;
    }

    get_event_settings();

    $dir_path = $esetting{'dir_path'};
    $sub_dir  = $esetting{'sub_dir'};
    $second   = substr($esetting{'time'}, 6, 2);

    if (!-e "$config_file") {
        print "\n\t$config_file not found!\n" if (($dbg)&&(!$quiet));
        print "\tsee 'motionmail.pl -man' for instructions.\n\n" if (!$quiet);
        exit;
    }

    $excl = get_settings();

    create_montage();

    if($excl) {
        # do not notify when the event was within an exclusion time block
        print "(".(get_date($now))[2].") Within exclusion time, not sending.\n" if (($dbg)&&(!$quiet));
        exit;
    } else {
        notify("$second-rmontage.png");

        unlink("$event_file") if ((!$dbg));
    }
}

sub create_montage {
    # create a montage from the images captured during this event
    unlink("$dir_path/$second-rmontage.png");
    unlink("$dir_path/$second-montage.png");

    opendir(DIR,$dir_path) || die "Can't open dir_path ($dir_path): $!\n"; 
    @files=readdir(DIR);
    close(DIR);

    @files=grep(!/montage/, @files);

    if ($setting{'montage_image_count'} == 0) {
        # default behaviour
        @newfiles=grep(/$second-/,@files);

    	# sort the files so the tiles appear in order 
        sort(@newfiles);
    } else {
        # advanced behaviour
        $sec_count = $second;

        while (@newfiles < $setting{'montage_image_count'}) {
            $ret=grep(/$sec_count-/,@files);
            push(@newfiles, grep(/$sec_count-/,@files));

            if($ret == 0) {
               # no more images found
               last;
            }

            $sec_count++;
        }

    	# sort the files so the tiles appear in order 
        sort(@newfiles);

        # limit the number of images to montage_image_count
        if (@newfiles > $setting{'montage_image_count'}) {
            @newfiles = @newfiles[0..$setting{'montage_image_count'} - 1];
        }
    }

    chdir $dir_path;
    $files=join " ",@newfiles;

    # failsave: if no images are found, don't try to run the montage command
    if(!$files){
        print "no files found, exiting...\n" if (($dbg)&&(!$quiet));
        exit;
    }
    # exec the montage command
    # command for email thumbnails
    $err = `montage -verbose -geometry 35% -adjoin $files $second-rmontage.png` if (($setting{'make_montage'})&&(!$excl));
    print $err if($dbg);

    # command for montage that gets left behind on the server
    $err = `montage -verbose -label %f -frame 5 -geometry 100% -adjoin $files $second-montage.png` if (($setting{'make_montage'})&&(!$excl));
    print $err if($dbg);

    # move event images to the history dir
    if($setting{'keep_history'}) {
        if(!-e "./history") { mkdir "./history", 0755; }
        foreach $file(@files) {
            $err = `mv --verbose $file ./history`;
            print $err if($dbg);
        }
    }
}

sub notify {
    my($f)=shift;
    #this stuff would be good to move into configuration varibales
    my $subject = "[" . hostname() . "] Motion detected on camera '" . $instance . "' at $esetting{'date'} $esetting{'time'}";
    my $message = "Motion was detected $esetting{'date'} $esetting{'time'}";

    send_mail($setting{'send_to'},$subject,$message,"$dir_path/$f");
}

sub send_mail {
    my($to,$subj,$message,$file)=@_;
    my $html=<<EOF;
<html>
<body>
  $message<br>
  <br>
  <a href="$setting{'server_url'}/$sub_dir/$second-montage.png"><img src="cid:event_$second.png"></a>
</body>
</html>
EOF

    use MIME::Lite;
    my $mime_msg = MIME::Lite->new( From    => "$setting{'app_name'} <$setting{'app_email'}>",
                                    To      => $to,
                                    CC      => $cc,
                                    Subject => $subj,
                                    'X-Application' => "motionmail $version by $setting{'author'}",
                                    Type    => 'multipart/related'
    ) || die("Error creating MIME message: $!");
    $mime_msg->attach( Type     => 'text/html',
                       Data     => $html );
    $mime_msg->attach( Type     => 'image/png',
                       Path     => $file,
                       Id       => "event_$second.png"
    ) if ($file);

    MIME::Lite->send('smtp', $setting{'mail_server'}) if ($setting{'mail_server'});
    $mime_msg->send() || die("Error sending message: $!");
    unlink $file;
}

sub get_settings {
    open(SETTINGS,"<$config_file") || die "Can't open $config_file: $!\n";
    @lines=<SETTINGS>;
    close(SETTINGS);

    # the following hash is only used to sort the settings properly
    # very ugly solution. it is only used with the verbose (-debug)
    # option so it probably can be removed altogether to reduce the
    # size of this script
    %wd=(
        'monday'     => 1,
        'tuesday'    => 2,
        'wednesday'  => 3,
        'thursday'   => 4,
        'friday'     => 5,
        'saturday'   => 6,
        'sunday'     => 7,
        'send_to'    => 8,
        'file_dir'   => 9,
        'server_url' => 10,
        'montage_image_count' => 11
    ); 

    # get rid of irrelevant lines
    @lines = grep(!/^#/,@lines);
    @lines = grep(!/^\s/,@lines);
    @lines = grep(!/^\n/,@lines);

  # fill setting hash with values from the settings.conf file
    foreach $line(@lines) {
        $line=~s/\s//g;
        ($key,$val)=split(/=/,$line);
        $setting{$key}=$val;
        if($val=~m/(\d{4})-(\d{4})/) {
            $begin{$key}=$1; $end{$key}=$2;
        }
    }

    # print a neatly formatted list of settings
    # (only with -debug)
    print "Settings:\n" if (($dbg)&&(!$quiet));
    for $key (sort { $wd{$a} <=> $wd{$b} } keys %setting) {
        printf ("%14s = %-40s\n",$key,$setting{$key}) if(($dbg)&&(!$quiet));
    }
    print "\n" if (($dbg)&&(!$quiet));

    # check if current event was detected within an exclusion time block
    # if it is, do not send anything. Events will still be saved.
    my($day,$time)=(get_date($now))[1,2];
    if(($time>=$begin{$day})&&($time<$end{$day})) {
        $dont_send=1;
    }
}

sub get_event_settings {
    open(ESETTINGS,'<', "$event_file") || die "Can't open $event_file: $!\n";
    @lines=<ESETTINGS>;
    close(ESETTINGS);

    %wd=(
        'dir_path'   => 1,
        'sub_dir'    => 2,
        'date'       => 3,
        'time'       => 4
    );

    # get rid of irrelevant lines
    @lines = grep(!/^#/,@lines);
    @lines = grep(!/^\s/,@lines);
    @lines = grep(!/^\n/,@lines);

    # fill setting hash with values from the settings.conf file
    foreach $line(@lines) {
        $line=~s/\s//g;
        ($key,$val)=split(/=/,$line);
        $esetting{$key}=$val;
    }

    # print a neatly formatted list of settings
    # (only with -debug)
    print "Event settings:\n" if (($dbg)&&(!$quiet));
    for $key (sort { $wd{$a} <=> $wd{$b} } keys %esetting) {
        printf ("%14s = %-40s\n",$key,$esetting{$key}) if(($dbg)&&(!$quiet));
    }
    print "\n" if (($dbg)&&(!$quiet));
}

sub get_date {
    my $time = shift;
        $time or ($time = time);
    my @weekday = qw/Sunday Monday Tuesday Wednesday Thursday Friday Saturday/;
    my ($min, $hr, $day, $mon, $yr, $wday) = (localtime($time))[1,2,3,4,5,6];
    $yr = $yr + 1900;
    $mon=$mon+1;
    ($min < 10) and ($min = "0$min");
    ($hr  < 10) and ($hr  = "0$hr");
    ($day < 10) and ($day = "0$day");
    ($mon < 10) and ($mon = "0$mon");
    return "$day$mon$yr","$weekday[$wday]","$hr$min","$weekday[$wday] $day/$mon/$yr $hr:$min";
}

__END__

=pod

=head1 NAME

motionmail - creates a montage from a collection of images and sends it
              to a list of email addresses.

=head1 SYNOPSIS

motionmail -e <EVENTNR> [-i <INSTANCE>] -p "<PATH-TO-PICTURES>" [-q]

=head1 OPTIONS

=over 8

=item B<-help> B<-h>

displays a brief help message

=item B<-man>

displays the full manual

=item B<-version> B<-V>

displays version information

=item B<-e> <EVENTNR>

sets the event number. motionmail will look for images for this event in
the image directory, it will then create a montage from these images and
send the montage to the email addresses defined in the motionmail.conf file

=item B<-i> <INSTANCE>

sets the instance number. motionmail will look for a configuration file
$start_path/motionmail-<INSTANCE>.conf at start-up instead of the default
file $start_path/motionmail.conf.
 
=item B<-p> <PATH-TO-PICTURES>

add sub directory path to default file_dir which has been set in the
configuration file.

=item B<-q>

quiet operation, this will prevent any output from being displayed. Be
aware that this will also prevent errors from being displayed.

=back

=head1 DESCRIPTION

motionmail will search for images that were stored by motion for the event
given. It will create a montage of these images and subsequently send this 
montage to the email addresses defined in $start_path/motionmail.conf.
The event images will be moved to a subdirectory called 'history', the 
montage image will be deleted after sending.
a separate cronjob should be defined to delete old events to prevent an
excess of data.

This script can be called by adding:
on_event_end /usr/local/bin/motionmail.pl -e=%v [-i %t] -p "%Y/%m/%d/%H/%M" [-q]

To your motion.conf


=head1 FILES

B<$start_path/motionmail[-<INSTANCE>].conf>

Configuration settings.
this file should contain the following:

 #---------- motionmail.conf ------------
 
 # settings file for motionmail
 # (see motionmail -man for instructions)
 # comment lines start with #
 # blank lines, lines starting with a space and
 # lines starting with # will be ignored
 
 # author of motionmail script
 author       = 'The Eisfair Team <team\@eisfair.org>'

 # comma separated list of email addresses
 send_to      = taco@cannex.com.au

 # directory where event images are stored by motion
 # (see the 'target_dir' setting in motion.conf)
 file_dir     = /home/cam/public_html

 # server url to access montage file on server
 server_url   = http://<my-server-fqdn>/motion/camera-<camera-number>

 # make the montage.png image (1=yes,0=no)
 # (nothing will be sent if this is set to 0)
 make_montage = 1

 # keep event images (1=yes,0=no)
 # this will store all event images in a subdir of
 # file_dir called 'history'
 keep_history = 1

 # application name
 # this name will appear in the 'From' field of all email
 app_name     = Motion

 # application email
 # this will be the sender address of all email
 app_email    = motion@cannex.com.au

 # the 'From:' field will be formed like this:
 # From: app_name <app_email>
 
 # for each weekday an exclusion time can be set
 # this is done in 24h notation in the form of <begin time hhmm>-<end time hhmm>,
 # events will only be sent outside the defined time block, when left empty
 # events will be sent for the full 24 hours. see example below
 
 monday    = 0800-1700
 tuesday   = 0800-1700
 wednesday = 0800-1700
 thursday  = 0800-1700
 friday    = 0800-1600
 saturday  =
 sunday    =
 
 #---------- motionmail.conf ------------

=head1 REQUIREMENTS

 Getopt::Long, Pod::Usage, MIME::Lite


=head1 VERSION

motionmail version 0.01 ( Mon Jun  6 08:03:03 EST 2005 )

=head1 AUTHOR

Taco Scheltema <taco@scheltema.org>, <taco@cannex.com.au>

=cut
